// https://www.cnblogs.com/lovellll/p/10173788.html
const request = require('request');

// let config = require('hff_config');

const fs = require("fs");
const path = require("path");

const Aes = require('hff_aes');

class Request {
    constructor() {
        this.config = this.getConfig();
        this.doc = this.getDoc();
        this.aes = new Aes(this.config.aes.key || "751f621ea5c8f931", this.config.aes.iv || "2624750004598719");
    }

    /**
     * 获取远程配置
     */
    async getRemoteConfig(name = 'config', form = {}) {
        const rs = await this.postApi({
            name,
            form
        });
        return rs
    }


    /**
     * 定时把本微服务注册到注册中心,并从注册中心取得最新全部已经注册微服务
     * @param {*} apiExplain 微服务api说明
     */
    async toRegistryInterval() {
        const intervalTime = this.config.intervalTime || 30000
        await this.toRegistry();
        setTimeout(() => {
            this.toRegistryInterval();
        }, intervalTime)
    }


    /**
     *  把本微服务注册到注册中心,并从注册中心取得最新全部已经注册微服务
     * 
     */
    async toRegistry() {
        const that = this;
        // 本微服务名字
        const name = that.config.name;
        // 注册中心地址
        let url = that.config.registry
        url += '/server';
        url = url.replace('//server', '/server');
        try {
            const rs = await this.postAes({
                url, form: {
                    // username: 'admin', // 其它参数
                    // password: 'admin', // 其它参数
                    // apiExplain: 'api说明一大堆~api说明一大堆~api说明一大堆~api说明一大堆~api说明一大堆~api说明一大堆~api说明一大堆~',    // 选填,api说明
                    apiExplain: that.config.apiExplain,    // 必填,api说明
                    doc: that.doc,
                    // name: config.name, // 必填,api名字 
                    name, // 必填,api名字 
                    port: that.config.ip,  // id。如果为空，ip会是自动获取的
                    port: that.config.port,  // 必填,端口
                    path: that.config.path   // 必填 访问路径.   http://192.168.1.2:18361/alisms中的 /alisms
                }
            });// 返回值
            // console.log(JSON.stringify(rs)) // 成功: {"code":200,"apiAddress":{"test3":{"apiExplain":"api说明~","item":["http://192.168.1.2:18361"]}}},  自己的地址错误，别人访问不了: {code: 500}

            if (rs.code === 200) {
                this.setApiAddress(rs.apiAddress);
            }

        } catch (e) {
            console.log(e)
        };

    }



    /**
     *  
     * @param {*} apiAddress  == 
        {
            "test3":{
                "apiExplain":"api说明~",
                "item":[
                    "http://192.168.1.2:18361"
                ]
            },
            "test4":{
                "apiExplain":"test4的api说明~",
                "item":[
                    "http://192.168.1.2:18362"
                ]
            },
            "test5":{
                "apiExplain":"test5的api说明~",
                "item":[
                    "http://192.168.1.2:18361"
                ]
            }
        
        }
     */
    setApiAddress(apiAddress) {
        // 设置负载均衡 count
        for (let name in apiAddress) {
            if(this.apiAddress && this.apiAddress[name] && this.apiAddress[name].hasOwnProperty('count')) {
                apiAddress[name].count = this.apiAddress[name].count;
            } else {
                apiAddress[name].count = Math.floor(Math.random() * apiAddress[name].item.length);
            }
        }
        this.apiAddress = apiAddress;
    }


    /**
     * 发送微服务请求,aes加密发送，返回的数据也是aes加密的。需要解密一下
     * @param {} param0 
     */
    async postApi({ name, path = '', form }) {
        let itemlength = this.apiAddress[name].item.length;
        // 循环一下，
        while (itemlength--) {
            try {
                // 负载均衡 count
                const count = (this.apiAddress[name].count++) % this.apiAddress[name].item.length;
                const url = this.apiAddress[name].item[count] + path;
                const rs = await this.postAes({ url, form });
                return rs
            } catch (e) {
                // return null;
            }
        }
        return null;
    }


    /**
     * 发送微服务请求,aes加密发送，返回的数据是普通的没有加密
     * @param {} param0 
     */
    async postCommonApi({ name, path = '', form }) {
        let itemlength = this.apiAddress[name].item.length;
        // 循环一下，
        while (itemlength--) {
            try {
                // 负载均衡 count
                const count = (this.apiAddress[name].count++) % this.apiAddress[name].item.length;
                const url = this.apiAddress[name].item[count] + path;
                const rs = await this.postCommonAes({ url, form });
                return rs
            } catch (e) {
                // return null;
            }
        }
        return null;
    }


    /**
     * 普通post请求,返回值是json对象
     * @param {*} param
     * url: url, //要请求的地址
     * form: {a: 'a', b: 'b'} //要请求的数据
     * @returns {json}  {c: 'c', d: 'd'}
     *
     */
    post({ url, form }) {
        return new Promise((resolve, reject) => {
            request.post({ url, form }, function (error, response, body) {
                if (error) {
                    // console.log(1)
                    reject({
                        code: 444,
                        error
                    });
                    return;
                }
                if (response.statusCode == 200) {
                    //    console.log(typeof body) // string
                    //    console.log(body) // 请求成功的处理逻辑  
                    // console.log(2)
                    resolve(JSON.parse(body))
                } else {
                    // console.log(3)
                    reject(body)
                }
            })
        })
    }


    /**
  * 普通post请求,返回值是字符串
  * @param {*} param
  * url: url, //要请求的地址
  * form: {a: 'a', b: 'b'} //要请求的数据
  * @returns {String}  "{c: 'c', d: 'd'}""
  *
  */
    postStr({ url, form }) {
        return new Promise((resolve, reject) => {
            request.post({ url, form }, function (error, response, body) {
                if (error) {
                    // console.log(1)
                    reject(error);
                    return;
                }
                if (response.statusCode == 200) {
                    //    console.log(typeof body) // string
                    //    console.log(body) // 请求成功的处理逻辑  
                    // console.log(2)
                    resolve(body)
                } else {
                    // console.log(3)
                    reject(body)
                }
            })
        })
    }

    /**
     * aes加密post请求,返回的数据也是加密的，需要解密
     * @param {*} param0 
     */
    async postAes({ url, form }) {
        // console.log(999);
        // console.log(form)
        const formStr = JSON.stringify(form);
        // console.log(9998);
        // console.log(`formStr:${formStr}`);
        const encryptStr = this.aes.encrypt(formStr);
        const body = await this.post({ url, form: { token: encryptStr } });
        // console.log(`body:${body}`)
        const str = this.aes.decrypt(body.token);
        // console.log(`str:${str}`)
        return JSON.parse(str);
    }

     /**
     * aes加密post请求,返回的数据是没有加密
     * @param {*} param0 
     */
    async postCommonAes({ url, form }) {
        // console.log(999);
        // console.log(form)
        const formStr = JSON.stringify(form);
        // console.log(9998);
        // console.log(`formStr:${formStr}`);
        const encryptStr = this.aes.encrypt(formStr);
        const body = await this.post({ url, form: { token: encryptStr } });
        // console.log(`body:${body}`)
        return body;
    }


    /**
     * api微服务端，解析post过来的Aes加密的数据， 就是上面postAes的解密
     * @param {*} that  controller里面的this
     */
    parsePostAes(that) {
        const token = that.post('token');
        if (!token) {
            return null;
        }
        // console.log('parseJwt,token:' + token)
        const checkData = this.aes.decrypt(token);
        // console.log(`checkData:`)
        // console.log(checkData)
        return JSON.parse(checkData);

    }

    /**
     * aes加密数据，以方便返回给客户端
     * @param {*} that constroller 里面的this
     * @param {*} form 要返回的数据，必须是json对象格式. {"a": "aa", "b": "bsdfsdf"} , [1, 2, "3"]
     */
    aesify(that, form) {
        const formStr = JSON.stringify(form);
        const encryptStr = this.aes.encrypt(formStr);
        that.body = { token: encryptStr }
    }


    getConfig(p = "./config.json") {
        const configPath = path.join(process.cwd(), p);
        let isConfigExist = fs.existsSync(configPath);
        let jsonConfig = null;

        try {
            if (isConfigExist) {
                jsonConfig = JSON.parse(fs.readFileSync(configPath, "utf8"));
            }
        } catch (e) {
            throw e;
        }

        return jsonConfig;
    }

    getDoc(p = "./doc.md") {
        const configPath = path.join(process.cwd(), p);
        let isConfigExist = fs.existsSync(configPath);
        let jsonConfig = null;

        try {
            if (isConfigExist) {
                jsonConfig = fs.readFileSync(configPath, "utf8");
            }
        } catch (e) {
            throw e;
        }

        return jsonConfig;
    }


    // js 对两个对象进行深层合并 ,obj2会覆盖obj1
    deepMerge(obj1, obj2) {
        let key;
        for (key in obj2) {
            // 如果target(也就是obj1[key])存在，且是对象的话再去调用deepMerge，否则就是obj1[key]里面没这个对象，需要与obj2[key]合并
            obj1[key] = obj1[key] && obj1[key].toString() === "[object Object]"
                ? this.deepMerge(obj1[key], obj2[key])
                : (obj1[key] = obj2[key]);
        }
        return obj1;
    }

    sleep (time) {
        return new Promise((resolve) => {
            setTimeout(() => {
                resolve();
            }, time)
        })
    }

}


module.exports = new Request()


    // const r = new Request();
    // r.toRegistry()



